@@ -1,6 +1,7 @@
 /**
  * @file dbus-server.h Purple DBUS Server
  * @ingroup core
+ * @see @ref dbus-server-signals
  */
 
 /* purple
@@ -22,16 +23,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
- *
- * @see @ref dbus-server-signals
  */
 
 #ifndef _PURPLE_DBUS_SERVER_H_
 #define _PURPLE_DBUS_SERVER_H_
 
+#include "dbus-purple.h"
 #include "value.h"
 
-
 G_BEGIN_DECLS
 
 /**
@@ -52,6 +51,8 @@ struct _PurpleDBusType {
     PurpleDBusType *parent;
 };
 
+#include "dbus-bindings.h"
+
 /* By convention, the PurpleDBusType variable representing each structure
    PurpleSomeStructure has the name PURPLE_DBUS_TYPE_PurpleSomeStructure.
    The following macros facilitate defining such variables
