@@ -169,7 +169,7 @@ msn_httpconn_parse_data(MsnHttpConn *httpconn, const char *buf,
 	/* Now we should be able to process the data. */
 	if ((s = purple_strcasestr(header, "X-MSN-Messenger: ")) != NULL)
 	{
-		char *full_session_id, *gw_ip, *session_action;
+		gchar *full_session_id = NULL, *gw_ip = NULL, *session_action = NULL;
 		char *t, *session_id;
 		char **elems, **cur, **tokens;
 
@@ -196,13 +196,16 @@ msn_httpconn_parse_data(MsnHttpConn *httpconn, const char *buf,
 		{
 			tokens = g_strsplit(*cur, "=", 2);
 
-			if (strcmp(tokens[0], "SessionID") == 0)
+			if (strcmp(tokens[0], "SessionID") == 0) {
+				g_free(full_session_id);
 				full_session_id = tokens[1];
-			else if (strcmp(tokens[0], "GW-IP") == 0)
+			} else if (strcmp(tokens[0], "GW-IP") == 0) {
+				g_free(gw_ip);
 				gw_ip = tokens[1];
-			else if (strcmp(tokens[0], "Session") == 0)
+			} else if (strcmp(tokens[0], "Session") == 0) {
+				g_free(session_action);
 				session_action = tokens[1];
-			else
+			} else
 				g_free(tokens[1]);
 
 			g_free(tokens[0]);
@@ -435,7 +438,7 @@ httpconn_write_cb(gpointer data, gint source, PurpleInputCondition cond)
 static gboolean
 write_raw(MsnHttpConn *httpconn, const char *data, size_t data_len)
 {
-	ssize_t res; /* result of the write operation */
+	gssize res; /* result of the write operation */
 
 	if (httpconn->tx_handler == 0)
 		res = write(httpconn->fd, data, data_len);
@@ -548,7 +551,7 @@ msn_httpconn_poll(gpointer data)
 	return TRUE;
 }
 
-ssize_t
+gssize
 msn_httpconn_write(MsnHttpConn *httpconn, const char *body, size_t body_len)
 {
 	char *params;
@@ -684,6 +687,17 @@ msn_httpconn_destroy(MsnHttpConn *httpconn)
 
 	g_free(httpconn->host);
 
+	while (httpconn->queue != NULL) {
+		MsnHttpQueueData *queue_data;
+
+		queue_data = (MsnHttpQueueData *) httpconn->queue->data;
+
+		httpconn->queue = g_list_delete_link(httpconn->queue, httpconn->queue);
+
+		g_free(queue_data->body);
+		g_free(queue_data);
+	}
+
 	purple_circ_buffer_destroy(httpconn->tx_buf);
 	if (httpconn->tx_handler > 0)
 		purple_input_remove(httpconn->tx_handler);
