@@ -29,6 +29,10 @@
 #include "libc_internal.h"
 #include <glib.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #ifdef _MSC_VER
 #define S_IRUSR S_IREAD
 #define S_IWUSR S_IWRITE
@@ -40,8 +44,8 @@
 #endif
 
 /* sys/socket.h */
-#define socket( namespace, style, protocol ) \
-wpurple_socket( namespace, style, protocol )
+#define socket( domain, style, protocol ) \
+wpurple_socket( domain, style, protocol )
 
 #define connect( socket, addr, length ) \
 wpurple_connect( socket, addr, length )
@@ -75,8 +79,8 @@ wpurple_send(socket, buf, buflen, flags)
 wpurple_ioctl( fd, command, val )
 
 /* fcntl.h */
-#define fcntl( fd, command, val ) \
-wpurple_fcntl( fd, command, val )
+#define fcntl( fd, command, ... ) \
+wpurple_fcntl( fd, command, ##__VA_ARGS__ )
 
 /* arpa/inet.h */
 #define inet_aton( name, addr ) \
@@ -98,6 +102,8 @@ wpurple_gethostbyname( name )
 wpurple_strerror( errno )
 #define strerror( errornum ) \
 wpurple_strerror( errornum )
+#define g_strerror( errornum ) \
+wpurple_strerror( errornum )
 
 /* unistd.h */
 #define read( fd, buf, buflen ) \
@@ -121,6 +127,8 @@ wpurple_g_access( filename, mode )
 #define gethostname( name, size ) \
 wpurple_gethostname( name, size )
 
+#define fsync(fd) _commit(fd)
+
 /* sys/time.h */
 #define gettimeofday( timeval, timezone ) \
 wpurple_gettimeofday( timeval, timezone )
@@ -151,4 +159,8 @@ wpurple_localtime_r( time, resultp )
 /* helper for purple_utf8_strftime() by way of purple_internal_strftime() in src/util.c */
 const char *wpurple_get_timezone_abbreviation(const struct tm *tm);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* _LIBC_INTERFACE_H_ */
